##############################################################################
# File:     makefile
# Date:     02/22/2007
# Purpose:  Builds the midrange PIC library and links it into a lib file
#
.SILENT:

#-----------------------------------------------------------------------------
# Project-specific settings. Define TARGET and SOURCE_FILES here. The target
# will be built from the asm files listed in the source files macro.
#
TARGET=main.hex
SOURCE_FILES = main.asm
LINK_LIBS=
MODULE_PATH=
#-----------------------------------------------------------------------------

##############################################################################
# Program paths, etc
#
ASM=/usr/local/bin/gpasm
LINK=/usr/local/bin/gplink
LIB=/usr/local/bin/gplib
PROCFILE=.processor
PROJECTINC=project.inc
PWD=$(shell pwd)
PROCESSOR?=$(strip $(shell if [ -a .processor ]; then cat .processor; else echo ;fi))

##############################################################################
# Program arguments for build environment
#
ASMFLAGS+= --expand ON -u -c
LINKFLAGS+=-m -O 0
LKRFILE=/usr/local/share/gputils/lkr/${PROCESSOR}_g.lkr
INCLUDEPATH=${MODULE_PATH:%=-I%}
LIBPATH=${MODULE_PATH:%=-I %/${PROCESSOR}}

##############################################################################
# Define object files, error and list files
#
HEXTARGET   =${TARGET:%.hex=${PROCESSOR}/%.hex}
LINK_OBJS   =${filter %.o,    ${SOURCE_FILES:%.asm=${PROCESSOR}/%.o}}
LST_FILES   =${filter %.lst , ${SOURCE_FILES:%.asm=${PROCESSOR}/%.lst}}

MAPFILES    =${filter %map,   ${SOURCE_FILES:%.asm=${PROCESSOR}/%.map}}
CODFILES    =${filter %.cod,  ${SOURCE_FILES:%.asm=${PROCESSOR}/%.cod}}
COFFILES    =${filter $.cof,  ${SOURCE_FILES:%.asm=${PROCESSOR}/%.cof}}
ERR_FILES   =${filter %.err , ${SOURCE_FILES:%.asm=${PROCESSOR}/%.err}}

all: ${PROJECTINC} ${LINK_OBJS}
	if [ -n "${VERBOSE}" ] ; then                                           \
  	echo ${LINK} ${LINKFLAGS} -s ${LKRFILE} ${LIBPATH}                      \
			-o ${HEXTARGET} ${LINK_OBJS} ${LINK_LIBS} ;                       \
        else                                                               \
	   echo "Linking $@..." ;                                               \
	fi
	${LINK} ${LINKFLAGS} -s ${LKRFILE} ${LIBPATH}                           \
			-o ${HEXTARGET} ${LINK_OBJS} ${LINK_LIBS}
   

##############################################################################
# PHONY setproc target. If the user provides a processor, overwrite the 
# processor file, otherwise do nothing.
# Creates the processor include file based upon the command line definition
# of PROC. The project include file is built in the main project directory
# and dictates the default action of make when a processor is not specified
# on the command line via make PROC=<processor>.
#
.PHONY: setproc
setproc:
	if [ -n "${PROC}" ] ;                                                 	\
	then                                                                    \
		echo "${PROC}" > ${PROCFILE};                                        \
	fi
     
##############################################################################
# To build the processor file, run the setproc target
#
${PROCFILE}: setproc     

##############################################################################
# Creates the processor include file based upon the command line definition
# of PROC.
#
${PROJECTINC}: ${PROCFILE}
	echo "Setting target architecture to ${PROCESSOR}..." ;                 \
	echo "; MAKE generated include file" > ${PROJECTINC};                   \
	echo "#ifndef __PROCESSOR_INCLUDE" >> ${PROJECTINC};                    \
	echo "#define __PROCESSOR_INCLUDE" >> ${PROJECTINC};                    \
	echo " ERRORLEVEL     -207" >> ${PROJECTINC};                           \
	echo " ERRORLEVEL     -302" >>  ${PROJECTINC};                          \
	echo " #include <P${PROCESSOR}.inc>" >> ${PROJECTINC};                  \
	echo "#endif" >> ${PROJECTINC};                                         \

##############################################################################
# Removes error and list files, leaving just the library file and the
# processor definition file.
#
postbuild: ${PROJECTINC}
	echo "Removing intermediate build files for ${PROCESSOR}"
	rm -rf ${LINK_OBJS} ${LST_FILES}
	rm -rf ${COFFILES} ${CODFILES} ${MAPFILES}

##############################################################################
# Removes all list and error files, as well as the target library
#
clean: ${PROJECTINC}
	echo "Removing object, list, COF, COD, MAP and hex files for ${PROCESSOR}"
	rm -rf ${LINK_OBJS} ${LST_FILES} 
	rm -rf ${COFFILES} ${CODFILES} ${MAPFILES} ${HEXTARGETS}

##############################################################################
# Removes all list, error, and target files along with the processor file
#
realclean:
	if [ -d ./${PROCESSOR} ] ;                                                 \
   then                                                                       \
		echo "Removing architecture build directory '${PROCESSOR}'" ;           \
      rm -rf ./${PROCESSOR} ;                                                 \
   fi
	echo "Removing configuration"
	rm -rf ${PROJECTINC} ${PROCFILE}

##############################################################################
# Build and object module from an assembler file
#
./${PROCESSOR}/%.o: %.asm
	if [ -z "${PROCESSOR}" ] ;                                                 \
   then                                                                       \
      echo "No Processor definition file found!" ;                            \
      echo "Remove ${PROJECTINC} and rebuild using make PROC=<processor>." ;  \
      exit -1 ;                                                               \
   fi
	if [ ! -d ./${PROCESSOR} ] ;                                               \
	then                                                                       \
	   mkdir -p ./${PROCESSOR} ;                                               \
   fi

	if [ -n "${VERBOSE}" ] ; then                                              \
	   echo ${ASM} ${ASMFLAGS} -p ${PROCESSOR} ${INCLUDEPATH} -o $@ $< ;       \
   else                                                                       \
	   echo "Assembling $<..." ;                                               \
   fi
	${ASM} ${ASMFLAGS} -p ${PROCESSOR} ${INCLUDEPATH} -o $@ $<

