;-----------------------------------------------------------------------------
; File:     ds1620.inc
; Date:		01/30/2017
; Purpose:
;
; $Author: jcleland $
;-----------------------------------------------------------------------------

;Include once
#ifndef		_DS1620_INC
	#define	_DS1620.INC

;.............................................................................
;<editor-fold defaultstate="collapsed" desc="Includes">
;Include util for delays
#include "util.inc"
;</editor-fold>

;.............................................................................
;<editor-fold defaultstate="collapsed" desc="Preprocessor">
;Constants declaration
DSPORT				equ	PORTA
DSTRIS				equ	(DSPORT|0x80)
DSDAT					equ	2
DSCLK					equ	1
DSRST					equ	0
;</editor-fold>

;.............................................................................
;<editor-fold defaultstate="collapsed" desc=".ds1620.data">
;Local data declaration
.ds1620.data      udata
ds1620loop			res	.2						;Loop counter register
ds1620cmd			res	.1						;Command data register
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_DS1620ToggleClk(port, pin)">
; Macro:    _DS1620ToggleClk
; Purpose:  Toggles the clock pin
_DS1620ToggleClk macro _port, _pin
	local ___set, ___clear, ___done
		banksel		_port
		btfss			_port, _pin
			goto		___set
		goto			___clear
___set
		bsf			_port, _pin
		goto			___done
___clear
		bcf			_port, _pin
___done
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_DS1620Init()">
; Macro:    _DS1620Init
; Purpose:  Configures the device port and sets the 3-wire bus in reset
_DS1620Init macro
		;Set the initial state for the bus: reset low and clock high. We don't
		; care how the data pin in configured at this point
		banksel		DSPORT
		movlw			(1<<DSRST)|(1<<DSCLK)|(1<<DSCLK)
		iorwf			DSPORT, f

		;Select the tristate register and configure all 3-wire pins as outputs
		banksel		DSTRIS
		movlw			~((1<<DSRST)|(1<<DSCLK)|(1<<DSCLK))
		clrf			PORTA
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_DS1620BusReset()">
; Macro:    _DS1620BusReset
; Purpose:  Resets the 3-wire bus state and terminates any active transaction
;				with the device. The reset line is left low, clock is left high,
;				the data line is unchanged.
_DS1620BusReset macro
		;Reset the initial state for the bus: reset low and clock high. We don't
		; care how the data pin in configured.
		banksel		DSPORT
		bcf			DSPORT,DSRST
		bcf			DSPORT,DSDAT
	_Delay2us
		bsf			DSPORT,DSCLK
	_Delay2us
		bsf			DSPORT,DSDAT
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_DS1620WriteCommand()">
; Macro:    _DS1620WriteCommand
; Purpose:  This macro writes the command specified
;				to the device. It does not reset the bus prior to sending the
;				command.
_DS1620WriteCommand macro _cmd
	local __bitLoop, __dataLow, __dataHigh, __clockData
		;Store the command in our register
		movlw			_cmd
		movwf			ds1620cmd

		;Load the loop register for 8 bits to send
		banksel		ds1620loop
		movlw			.8
		movwf			ds1620loop

		;Select the port bank and drive the reset pin high to begin
		; communication
		banksel		DSPORT
		bsf			DSPORT,DSRST
	_Delay2us
		bcf			DSPORT,DSCLK
	_Delay2us

__bitLoop
		;Send the command data LSb first, branch on bit 0. We'll shift through
		; each bit so what is bit 1 will be bit 0 on the next iteration. We'll
		; do this 8 times.
		banksel		ds1620cmd
		btfss			ds1620cmd,0
			goto		__dataLow
		goto			__dataHigh

__dataLow
		banksel		DSPORT
		bcf			DSPORT,DSDAT
		goto			__clockData

__dataHigh
		banksel		DSPORT
		bsf			DSPORT,DSDAT

__clockData

		;Now that the data pin is set to 0 or 1 for the current bit, strobe the
		; clock line for 2 microseconds, allowing the device to read the current
		; data bit.
		bcf			DSPORT,DSCLK
	_Delay2us
		bsf			DSPORT,DSCLK

		;Rotate the command byte right, shifting bit 0 into the carry flag. All
		; bits move down one position so bit 1 is now bit 0 and soforth.
		banksel		ds1620cmd
		rrf			ds1620cmd, f

		;Check the counter to see if we've sent all 8 bits. If not, loop back
		; to __bitLoop and send the next bit in the command byte.
		banksel		ds1620loop
		decfsz		ds1620loop, f
			goto		__bitLoop

		banksel		DSPORT
		bsf			DSPORT, DSCLK
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_DS1620ReadWordData(buffer)">
; Macro:    _DS1620ReadWordData
; Purpose:  This macro reads word data from the device into the ds1620data
;				file register. The data is read LSB first, but stored MSB
_DS1620ReadWordData macro _buffer
	local __byteLoop, __bitLoop
		bankisel		_buffer
		movlw			_buffer
		movwf			FSR
		clrf			INDF

		banksel		DSTRIS						;Select tristate register bank
		bsf			DSTRIS,DSDAT				; and switch data pin to input

		;Load the outer loop register for 2 bytes. We don't really need to read
		; the entire second byte, but we're having trouble with unstable
		; readings returned from the device so we're going to try it...
		banksel		ds1620loop+1
		movlw			.2
		movwf			ds1620loop+1

__byteLoop
		banksel		ds1620loop					;Select loop buffer bank
		movlw			.8								;Load for 8 bit read in low byte
		movwf			ds1620loop					;

__bitLoop

	  _Delay4us
		banksel		DSPORT						;Clock low to read first byte
		bcf			DSPORT, DSCLK
	  _Delay2us

		bsf			STATUS, C
		btfss			DSPORT,DSDAT
			bcf		STATUS, C
		rrf			INDF, f						; rotate carry flag into data

	  _Delay4us
		bsf			DSPORT, DSCLK

		banksel		ds1620loop
		decfsz		ds1620loop, f
			goto		__bitLoop

		;Decrement the register pointer for MSB
	  _Delay2us
		incf			FSR, f
	   clrf			INDF

		banksel		ds1620loop+1
		decfsz		ds1620loop+1, f
			goto		__byteLoop

	  _Delay2us
		banksel		DSPORT						;Clock low to read first byte
		bsf			DSPORT,DSCLK				;Set clock low to read bit

		banksel		DSTRIS						;Select tristate register bank
		bcf			DSTRIS,DSDAT				; and switch data pin to input
	endm
;</editor-fold>

;Include once
#endif