;-----------------------------------------------------------------------------
; File:     main.asm
; Date:		01/30/2017
; Purpose:  This module programs and reads temperature from a maxim DS1620
;           thermostat temperature device and displays the value on a
;           Hitachi HD44780 LCD display.
;
; $Author: jcleland $
; $Log$
;-----------------------------------------------------------------------------

;Disable benign warnings
   ERRORLEVEL     -207
   ERRORLEVEL     -302

;.............................................................................
;<editor-fold defaultstate="collapsed" desc="Config Bits">
;Set configuration bits
;
  __CONFIG(_PWRTE_OFF&_INTRC_OSC_NOCLKOUT&_BOR_OFF&_WDT_OFF&_MCLRE_OFF&_CP_OFF&_IESO_OFF&_FCMEN_OFF)
;</editor-fold>

;.............................................................................
;<editor-fold defaultstate="collapsed" desc="Preprocessor">
;Define oscillator configuration bits for OSCON register
#define           __OSCCONFREQBITS     b'01110000'

;Status register bits
#define           HALFDEGREE           0
#define           NEGATIVE             1
;</editor-fold>

;.............................................................................
; <editor-fold defaultstate="collapsed" desc="Includes">
; Includes

;Include processor header
#include "P16F690.inc"

;Include local headers
#include "util.inc"
#include "ds1620.inc"
#include "hitachi.hd44780.inc"
;</editor-fold>

;.............................................................................
;<editor-fold defaultstate="collapsed" desc=".main.data ">
;Local data declaration
.main.data        udata
mainStat          res   .1                ;Status register, just for .5 now
mainDigits        res   .3                ;Receive buffer
mainByte          res   .2                ;
mainTemp          res   .1
;</editor-fold>

;=============================================================================
;<editor-fold defaultstate="collapsed" desc="Program Memory Start">
; Program start jumps to Start label
;
.romStart         code 0x00
      goto        Start                   ;Startup jumps to Start label

;-----------------------------------------------------------------------------
;Jump to interrupt handler
;
.romInterrupt     code 0x04               ;Jump to interrupt handler from here
      goto        Interrupt

;-----------------------------------------------------------------------------
;Begin code section
;
.main.code      code


;-----------------------------------------------------------------------------
;String definitions
;
TEMP              dt    "Current Temp:",0
POSITIVETEMP      dt    "+",0
NEGATIVETEMP      dt    "-",0
POINT5            dt    ".5",0
DEGREESC          dt    b'11011111',"C  ",0
;</editor-fold>

;=============================================================================
; <editor-fold defaultstate="collapsed" desc="Start">
; Subroutine:  Start
; Purpose:
Start:
      ;Global Interrupt disable while configuring device. Notice that we need
      ; to loop this operation, testing the GIE bit of INTCON to be sure it is
      ; clear. The INTCON register is available across all 4 data memory banks
      ; and thus, we don't need to change bank selection bits to address it.
__WaitGIE_Disable
      bcf         INTCON,GIE              ;Clear the GIE bit of interrupt config
      btfsc       INTCON,GIE              ;Wait for bit to clear, and
       goto       __WaitGIE_Disable       ; loop while still set

      ;Disable peripheral interrupts
      bcf         INTCON,PEIE             ;Disable all peripheral interrupts

      ;Oscillator configuration. Note the use of a frequency bits macro here
      ; that is defined above specifically for the PIC16F690 device. We're
      ; configuring the device to use the internal oscillator at 4MHz. The
      ; Final two instructions in this section test for a stable oscillator
      ; before continuing.
      banksel     OSCCON                  ;Select bank
      clrf        OSCCON                  ; unset freq bits to start at 000
      movlw       __OSCCONFREQBITS        ;Load bits into accumulator
      iorwf       OSCCON, f               ; or with current bank settings
      bsf         OSCCON,SCS              ;Set for two-step startup
      bcf         OSCCON, 3

      banksel     OSCTUNE
      movlw       b'00000000'
      movwf       OSCTUNE

___Start_htswait
      btfss       OSCCON, HTS             ;Test for stable oscillator
       goto       ___Start_htswait        ; still clear, keep looping

;      bsf         INTCON,PEIE             ;Enable all peripheral interrupts
;      bsf         INTCON,GIE              ;Enable interrupts globally

      ;Be sure to set all pins to digital IO
      banksel     ANSELH
      clrf        ANSELH
      banksel     ANSEL
      clrf        ANSEL

      banksel     WPUA
;      movlw       b'00000000'
;      movwf       WPUA
      movlw       b'11111111'
      movwf       WPUA

   ;__TuneOscillator PORTA, 5

     _DS1620Init                          ;Initialize the DS1620

     _lcdSetCursor 0

     _lcdInitDisplay                    ;Initialize the display
     _lcdDelay60ms
     _lcdWriteTable TEMP


; </editor-fold>

;=============================================================================
; <editor-fold defaultstate="collapsed" desc="Main">
; Subroutine:  Start
; Purpose:
Main:

      ;This block of code issues two command bytes: The first indicates that
      ; there is a configuration command to follow and the second tells the
      ; DS1620 to operate in continuous conversion mode.
     _DS1620WriteCommand 0x0c

     _DS1620WriteCommand 0x00

     _DS1620BusReset

     _DS1620WriteCommand 0xee

     _DS1620BusReset
___Main_loop
;     _DS1620WriteCommand 0xee
;     _Delay250us

;     _DS1620BusReset

     _lcdDelay60ms

;     _DS1620WriteCommand 0x22

;     _DS1620BusReset

     _DS1620WriteCommand 0xaa
     _Delay100us

	  _memset mainByte, 0x00, .2
     _DS1620ReadWordData mainByte

     _DS1620BusReset
;</editor-fold>
;.............................................................................
; <editor-fold defaultstate="collapsed" desc="LCDOutputTemperature">
; Subroutine:  Start
; Purpose:
LCDOutputTemperature:
      ;_______________________________________________________________________
      banksel     mainTemp
      movf        mainByte, w
      movwf       mainTemp
     _byteOut     mainTemp, PORTA, 4, 5
      ;_______________________________________________________________________

     _memset mainDigits, 0x00, .3
      clrc                                ;Clear carry for shifting
      banksel     mainByte
      rrf         mainByte+1, f           ;Shift 9th bit into carry
      rrf         mainByte, f             ; carry into temp


      ;Check carry flag for .5 degree bit
      banksel     mainStat
      clrf        mainStat
      btfsc       STATUS, C
         bsf      mainStat, HALFDEGREE

      ;Check mainByte to see if MSb is set indicating negative
      btfsc       mainByte, 7
         bsf      mainStat, NEGATIVE

      ;Perform byte to digits conversion and set cursor position
     _lcdByteToInt mainByte, mainDigits
     _lcdSetCursor 0x40

      ;Handle negative sign
      banksel     mainStat
      btfss       mainStat, NEGATIVE
         goto     ___PositiveTemp

___NegativeTemp
      _lcdWriteTable NEGATIVETEMP
      goto        ___WriteTemp

___PositiveTemp
      _lcdWriteTable POSITIVETEMP
      goto        ___WriteTemp

___WriteTemp
      ;Write temperature digits
     _lcdWriteInt mainDigits, 3

      ;Handle .5 degrees
      banksel     mainStat
      btfss       mainStat, HALFDEGREE
         goto     ___writeDegreesC

      _lcdWriteTable POINT5

___writeDegreesC
     _lcdWriteTable DEGREESC

      goto        ___Main_loop            ;Back to main, start over
; </editor-fold>

;=============================================================================
; <editor-fold defaultstate="collapsed" desc="Interrupt">
; Function:    Interrupt
; Purpose:
Interrupt:
   retfie                                 ;Return from interrupt handler
; </editor-fold>

   end ;Stop assembly
